package mvvm;

import android.text.TextUtils;
import android.util.Log;

import androidx.annotation.NonNull;

import com.kaadas.lock.base.mvvm.BaseViewModel;
import com.kaadas.lock.ble.BleManager;
import com.kaadas.lock.ble.api.DeviceConnector;
import com.kaadas.lock.ble.callback.PasswordFactorCallback;
import com.kaadas.lock.ble.core.DeviceConnectorProvider;
import com.kaadas.lock.ble.core.callback.ConnectDeviceCallback;
import com.kaadas.lock.ble.core.callback.ConnectStateCallback;
import com.kaadas.lock.ble.core.callback.EnableNotifyCallBack;
import com.kaadas.lock.ble.core.scan.ScanConfig;
import com.kaadas.lock.ble.core.scan.ScanDeviceCallback;
import com.kaadas.lock.ble.core.scan.ScanResultCallback;
import com.kaadas.lock.ble.domain.DefaultConnConfig;
import com.kaadas.lock.ble.model.BleDevice;
import com.kaadas.lock.ble.utils.UuidUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 简单介绍一下：蓝牙的基础用法
 * author: prophet
 * date: 2022/10/27
 */
public class AbsBaseBleViewModel extends BaseViewModel {

    public SingleLiveEvent2<BleDevice> scanDeviceLiveData = new SingleLiveEvent2<>();
    private final SingleLiveEvent2<BLEStateEvent> bleState = new SingleLiveEvent2<>();
    public SingleLiveEvent2<Boolean> receivePasswordFactorLiveData = new SingleLiveEvent2<>();
    private Map<String, BleDevice> macString = new HashMap<>();

    public SingleLiveEvent2<BleDevice> getScanDeviceLiveData() {
        return scanDeviceLiveData;
    }

    public void setScanDeviceLiveData(SingleLiveEvent2<BleDevice> scanDeviceLiveData) {
        this.scanDeviceLiveData = scanDeviceLiveData;
    }

    public SingleLiveEvent2<BLEStateEvent> getBleState() {
        return bleState;
    }

    public SingleLiveEvent2<Boolean> getReceivePasswordFactorLiveData() {
        return receivePasswordFactorLiveData;
    }


    /**
     * 盲扫蓝牙   根据蓝牙名进行过滤
     *
     * @param name
     */
    public void scanBluetoothByDeviceName(long scanTime, String name) {
        macString.clear();
        BleManager.getInstance().stopScanDevices();
        ScanConfig scanConfig = new ScanConfig()
                .setTimeout(scanTime);//设置扫码超时时间
        BleManager.getInstance().scanDevices(scanConfig, new ScanResultCallback() {
            @Override
            public void onScanFinished(List<BleDevice> list, int i) {
                Log.e("fff", "onScanFinished:" + i);
                if (i == ScanDeviceCallback.SCAN_FINISHED_TIMEOUT) { //扫码失败  用连接失败的状态标志
                    bleState.setValue(new BLEStateEvent(BLEStateEvent.CONNECT_FAIL));
                }
            }

            @Override
            public void onFoundDevice(BleDevice bleDevice) {
                Log.e("fff", "扫描蓝牙：" + bleDevice.getDeviceName() + "   mac:" + bleDevice.getDeviceMAC());
                if (!TextUtils.isEmpty(bleDevice.getDeviceMAC()) && macString.get(bleDevice.getDeviceMAC()) == null
                        && !TextUtils.isEmpty(bleDevice.getDeviceName()) && bleDevice.getDeviceName().contains(name)) {
                    macString.put(bleDevice.getDeviceMAC(), bleDevice);
                    scanDeviceLiveData.postValue(bleDevice);
                }
            }

            @Override
            public void onMatchDevice(BleDevice bleDevice) {
            }

            @Override
            public void onScanFail(int errorCode) {
                super.onScanFail(errorCode);
                Log.e("fff", "onScanFail:" + errorCode);
                bleState.setValue(new BLEStateEvent(BLEStateEvent.CONNECT_FAIL));//扫码失败  用连接失败的状态标志
            }
        });
    }

    /**
     * 通过mac地址扫码蓝牙
     *
     * @param mac
     * @param scanTime
     */
    public void scanBluetoothByMac(String mac, long scanTime) {
        macString.clear();
        BleManager.getInstance().stopScanDevices();
        ScanConfig scanConfig = new ScanConfig()
                .setTimeout(scanTime)//设置扫码超时时间
                .setDeviceMac(mac);
        BleManager.getInstance().scanDevices(scanConfig, new ScanResultCallback() {
            @Override
            public void onScanFinished(List<BleDevice> list, int i) {
                if (i == ScanDeviceCallback.SCAN_FINISHED_TIMEOUT) { //扫描失败  用连接失败的状态标志
                    bleState.setValue(new BLEStateEvent(BLEStateEvent.CONNECT_FAIL));
                }
            }

            @Override
            public void onFoundDevice(BleDevice bleDevice) {
                if (!TextUtils.isEmpty(bleDevice.getDeviceMAC()) && macString.get(bleDevice.getDeviceMAC()) == null
                        && TextUtils.equals(bleDevice.getDeviceMAC(), mac)) {
                    macString.put(bleDevice.getDeviceMAC(), bleDevice);
                    scanDeviceLiveData.postValue(bleDevice);
                }
            }

            @Override
            public void onMatchDevice(BleDevice bleDevice) {
            }

            @Override
            public void onScanFail(int errorCode) {
                super.onScanFail(errorCode);
                bleState.setValue(new BLEStateEvent(BLEStateEvent.CONNECT_FAIL));//扫码失败  用连接失败的状态标志
            }
        });
    }

    /**
     * 连接蓝牙设备
     */
    public void connectBle(BleDevice bleDevice) {
        if (bleDevice == null) {
            bleState.setValue(new BLEStateEvent(BLEStateEvent.CONNECT_FAIL));
            return;
        }
        DefaultConnConfig defaultConnConfig = new DefaultConnConfig();
        bleDevice.setAutoEnableNotify(true);
        bleDevice.setMtu(251);
        BleManager.getInstance().releaseAll();
        DeviceConnectorProvider.clearCache(bleDevice.getDeviceMAC());
        BleManager.getInstance().setConnectConfig(bleDevice, defaultConnConfig);
        BleManager.getInstance().connectDevice(bleDevice, bleConnectCallback);
    }

    /**
     * 蓝牙连接结果回调
     */
    private final ConnectDeviceCallback bleConnectCallback = new ConnectDeviceCallback() {

        @Override
        public void onStartConnect(DeviceConnector connector) {
            ConnectDeviceCallback.super.onStartConnect(connector);
            if (connector != null && connector.getBleWifiController() != null) {
                connector.getBleWifiController().receivePasswordFactor(new PasswordFactorCallback() {
                    @Override
                    public void onPwdFactor(int i, byte[] bytes, boolean b) {
                        receivePasswordFactorLiveData.postValue(b);
                    }
                });
            }
        }

        @Override
        public void onConnectSuccess(DeviceConnector connector) {
//            connector.subscribeConnectState(connectStateCallback);
            bleState.setValue(new BLEStateEvent(BLEStateEvent.CONNECT_SUCCESS));
            if (connector != null) {
                connector.enableNotify(UuidUtils.DISTRIBUTION_NETWORK_NOTIFY_CHAR, true, new EnableNotifyCallBack() {
                    @Override
                    public void onEnableNotifyResult(boolean isSuccess, int status, String serviceUuid, String value) {

                    }
                });
            }
        }

        @Override
        public void onConnectFail(DeviceConnector connector, int errorCode) {
            Log.e("ble", "onConnectFail:" + errorCode);
            bleState.postValue(new BLEStateEvent(BLEStateEvent.CONNECT_FAIL));
            BleManager.getInstance().setCurrentBleDevice(null);
        }


        @Override
        public void onAuthSuccess(DeviceConnector connector) {
            ConnectDeviceCallback.super.onAuthSuccess(connector);
        }

        @Override
        public void onAuthFail(DeviceConnector connector, int errorCode) {
            ConnectDeviceCallback.super.onAuthFail(connector, errorCode);
        }

        @Override
        public void onDisconnected(DeviceConnector connector) {
            Log.e("ble", "onDisconnected:");
            bleState.postValue(new BLEStateEvent(BLEStateEvent.DISCONNECTED));
        }
    };

    /**
     * 蓝牙连接状态监听
     */
    private final ConnectStateCallback connectStateCallback = new ConnectStateCallback() {
        @Override
        public void onConnectStateChange(BleDevice bleDevice, boolean state) {
            if (!state) {
                Log.e("ble", "蓝牙断开连接");
                bleState.postValue(new BLEStateEvent(BLEStateEvent.DISCONNECTED));
            }
        }
    };

    /**
     * 判断当前mac地址的蓝牙是否连接
     *
     * @param mac
     * @return
     */
    public boolean isBleConnect(@NonNull String mac) {
        return BleManager.getInstance().isConnected(mac);
    }

    /**
     * 判断当前mac地址的蓝牙是否鉴权
     *
     * @param mac
     * @return
     */
    public boolean isBleAuth(@NonNull String mac) {
        DeviceConnector connector = BleManager.getInstance().getDeviceConnector(mac);
        return connector != null && connector.isAuth();
    }

    public boolean isBleOpen() {
        return BleManager.getInstance().isBleEnable();
    }

    public void stopScan() {
        BleManager.getInstance().stopScanDevices();
    }


    public void releaseBluetooth() {
        Log.e("bleWifi", "abs 蓝牙释放");
        BleManager.getInstance().releaseAll();
    }


    @Override
    protected void onCleared() {
        super.onCleared();
        Log.e("bleWifi", "abs onCleared蓝牙释放");
        BleManager.getInstance().releaseAll();
    }
}
